#!/bin/bash
timeLim=9000
args="-rnd-phase -no-clean-exit"
solver=penelope
maxSatELiteTime=300

if [ "x$1" = "x" ]; then
  echo "USAGE: solver.sh <solver> <input CNF>"
  exit 1
fi

# to set in evaluation environment
mypath="$(dirname "$(readlink -f ${BASH_SOURCE[0]})")"

# To set in a normal envirnement
#if [ -z "$TMPDIR" ]; then
    echo "c setting TMPDIR to $mypath/tmp"
    TMPDIR=$mypath/tmp
    mkdir -p $TMPDIR
#fi

(sleep $maxSatELiteTime; st=`ps | grep "SatELite" | sed 's/^ *//;s/ *$//'| cut -d' ' -f1`; echo "c $st";if [ "$st" != "" ]; then kill $st; fi;) &

TMP=$TMPDIR/penelope_$$ #set this to the location of temporary files
Satelite=./SatELite_release           #set this to the executable of SatELite
INPUT=$1;
shift 
echo "c"
echo "c Starting SatElite Preprocessing"
echo "c"
SatOutput=`$Satelite $INPUT $TMP.cnf $TMP.vmap $TMP.elim`
X=$?
echo "$SatOutput"

#if Satelite found the answer, we may stop now
if [ $X == 20 ]
then
  exit $X
fi

timeSatelite=`echo "$SatOutput" | grep "c CPU time:" | sed 's/.* \([0-9]*\.[0-9]*\).*/\1/'`
if [ "x$timeSatelite" = "x" ]
then
	#if there was too many clauses
	timeSatelite="0"
fi
echo "c $timeSatelite"
timeSatelite=`echo "$timeLim - $timeSatelite" | bc | awk '{print int($1)}'`
args=$args" -time-lim="$timeSatelite

echo "c"
echo "c Starting the solver"
echo "c"
echo "c "$args
if [ $X == 0 ]; then

  emptyResult=`more $TMP.cnf | grep "p cnf 0 0"`  
  if [ "$emptyResult" == "p cnf 0 0" ]; then
    ./$solver $INPUT $args "-force-print"
    X=$?
    exit $X
  fi
  #SatElite terminated correctly

  
  ./$solver $TMP.cnf $args $TMP.result
  exitCode=$?
  if [ $exitCode == 20 ]; then
    echo "s UNSATISFIABLE"
    rm -f $TMP.cnf $TMP.vmap $TMP.elim $TMP.result
    exit 20
    #Don't call SatElite for model extension.
  elif [ $exitCode != 10 ]; then
    #timeout/unknown, nothing to do, just clean up and exit.
    rm -f $TMP.cnf $TMP.vmap $TMP.elim $TMP.result
    exit $exitCode
  fi

  #SATISFIABLE, call SatElite for model extension
  echo "c calling SatElite for model extension of " $INPUT
  $Satelite +ext $INPUT $TMP.result $TMP.vmap $TMP.elim 
  X=$?
elif [ $X == 11 ]; then
  #SatElite died, penelope must take care of the rest
  
  ./$solver $INPUT $args "-force-print" #but we must force glucose to print out result here!!!
  X=$?
elif [ $X == 12 ]; then
  #SatElite prints out usage message
  #There is nothing to do here.
  X=0
else
  echo "c unknown return: "$X
  echo "c maybe SatELite was killed?"

  ./$solver $INPUT $args "-force-print"
  X=$?
fi

rm -f $TMP.cnf $TMP.vmap $TMP.elim $TMP.result $TMP.cnf
exit $X
